#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.24 02:00:00                  #
# ================================================== #

from PySide6 import QtCore
from PySide6.QtGui import QStandardItemModel
from PySide6.QtWidgets import QWidget, QHBoxLayout, QPushButton, QVBoxLayout, QLabel, QCheckBox

from pygpt_net.ui.widget.lists.base import BaseList
from pygpt_net.utils import trans
import pygpt_net.icons_rc

class ModelImporter(QWidget):
    def __init__(self, window=None):
        """
        Models importer widget

        :param window: main window
        """
        super(ModelImporter, self).__init__(window)
        self.window = window
        self.id = "models.importer"
        self.layout = self.setup()
        self.window.ui.models["models.importer.available"] = self.create_model(self.window)
        self.window.ui.nodes["models.importer.available"].setModel(self.window.ui.models["models.importer.available"])
        self.window.ui.nodes["models.importer.available"].selectionModel().selectionChanged.connect(
            lambda: self.window.controller.model.importer.change_available()
        )
        self.window.ui.models["models.importer.current"] = self.create_model(self.window)
        self.window.ui.nodes["models.importer.current"].setModel(self.window.ui.models["models.importer.current"])
        self.window.ui.nodes["models.importer.current"].selectionModel().selectionChanged.connect(
            lambda: self.window.controller.model.importer.change_current()
        )
        self.setLayout(self.layout)

    def create_model(self, parent) -> QStandardItemModel:
        """
        Create model

        :param parent: parent widget
        :return: QStandardItemModel
        """
        return QStandardItemModel(0, 1, parent)

    def setup(self):
        """Setup layout"""
        layout = QHBoxLayout()
        arrows_layout = QVBoxLayout()
        self.window.ui.nodes["models.importer.add"] = QPushButton(">")
        self.window.ui.nodes["models.importer.add"].clicked.connect(
            lambda: self.window.controller.model.importer.add()
        )
        self.window.ui.nodes["models.importer.remove"] = QPushButton("<")
        self.window.ui.nodes["models.importer.remove"].clicked.connect(
            lambda: self.window.controller.model.importer.remove()
        )
        self.window.ui.nodes["models.importer.add"].setEnabled(False)  # initially disabled
        self.window.ui.nodes["models.importer.remove"].setEnabled(False)  # initially disabled
        arrows_layout.addWidget(self.window.ui.nodes["models.importer.add"])
        arrows_layout.addWidget(self.window.ui.nodes["models.importer.remove"])

        self.window.ui.nodes["models.importer.available.label"] = QLabel(trans("models.importer.available.label"))
        self.window.ui.nodes["models.importer.available"] = BaseList(self.window)
        self.window.ui.nodes["models.importer.available"].clicked.disconnect()

        self.window.ui.nodes["models.importer.current.label"] = QLabel(trans("models.importer.current.label"))
        self.window.ui.nodes["models.importer.current"] = BaseList(self.window)
        self.window.ui.nodes["models.importer.current"].clicked.disconnect()

        self.window.ui.nodes["models.importer.available.all"] = QCheckBox(trans("models.importer.all"), self.window)
        self.window.ui.nodes["models.importer.available.all"].clicked.connect(
            lambda: self.window.controller.model.importer.toggle_all(
                self.window.ui.nodes["models.importer.available.all"].isChecked())
        )

        available_layout = QVBoxLayout()
        available_layout.addWidget(self.window.ui.nodes["models.importer.available.label"])
        available_layout.addWidget(self.window.ui.nodes["models.importer.available"])
        available_layout.addWidget(self.window.ui.nodes["models.importer.available.all"])

        selected_layout = QVBoxLayout()
        selected_layout.addWidget(self.window.ui.nodes["models.importer.current.label"])
        selected_layout.addWidget(self.window.ui.nodes["models.importer.current"])

        layout.addLayout(available_layout)
        layout.addLayout(arrows_layout)
        layout.addLayout(selected_layout)
        return layout

    def update_available(self, data):
        """
        Update available models

        :param data: Data to update
        """
        id = "models.importer.available"
        self.window.ui.nodes[id].backup_selection()
        self.window.ui.models[id].removeRows(0, self.window.ui.models[id].rowCount())
        i = 0
        for n in data:
            self.window.ui.models[id].insertRow(i)
            name = data[n].id
            if data[n].name != data[n].id:
                name += f" --> {data[n].name}"
            index = self.window.ui.models[id].index(i, 0)
            tooltip = data[n].id
            self.window.ui.models[id].setData(index, tooltip, QtCore.Qt.ToolTipRole)
            self.window.ui.models[id].setData(self.window.ui.models[id].index(i, 0), name)
            i += 1
        self.window.ui.nodes[id].restore_selection()

    def update_current(self, data):
        """
        Update current models

        :param data: Data to update
        """
        id = "models.importer.current"
        self.window.ui.nodes[id].backup_selection()
        self.window.ui.models[id].removeRows(0, self.window.ui.models[id].rowCount())
        i = 0
        for n in data:
            self.window.ui.models[id].insertRow(i)
            name = data[n].id
            if data[n].name != data[n].id:
                name += f" --> {data[n].name}"
            if data[n].imported:
                name = "* " + name  # mark imported models
            index = self.window.ui.models[id].index(i, 0)
            tooltip = data[n].id
            self.window.ui.models[id].setData(index, tooltip, QtCore.Qt.ToolTipRole)
            self.window.ui.models[id].setData(self.window.ui.models[id].index(i, 0), name)
            i += 1
        self.window.ui.nodes[id].restore_selection()

    def update_lists(self, available: dict, current: dict):
        """
        Update lists

        :param available: data with available models
        :param current: data with current models
        """
        self.update_available(available)
        self.update_current(current)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.30 02:00:00                  #
# ================================================== #

from typing import Any, Dict, List


class CheckboxList:
    def __init__(self, window=None):
        """
        CheckboxList field handler

        :param window: Window instance
        """
        self.window = window

    def apply(
            self,
            parent_id: str,
            key: str,
            option: Dict[str, Any]
    ):
        """
        Apply values to checkboxes

        :param parent_id: Options parent ID
        :param key: Option key
        :param option: Option data
        """
        if "value" not in option:
            return
        value = option["value"]
        exploded_list = value.split(",") if isinstance(value, str) else []
        if key not in self.window.ui.config[parent_id]:
            self.window.ui.config[parent_id][key] = {}
        for item in self.window.ui.config[parent_id][key].boxes:
            if self.window.ui.config[parent_id][key].boxes[item] is not None:
                self.window.ui.config[parent_id][key].boxes[item].setChecked(False)
        for item in exploded_list:
            item = item.strip()
            if item not in self.window.ui.config[parent_id][key].boxes:
                continue
            if self.window.ui.config[parent_id][key].boxes[item] is None:
                continue
            self.window.ui.config[parent_id][key].boxes[item].setChecked(True)

    def on_update(
            self,
            parent_id: str,
            key: str,
            option: dict,
            value: Any,
            subkey: str = None,
            hooks: bool = True
    ):
        """
        Event: on update checkbox value

        :param parent_id: Options parent ID
        :param key: Option key
        :param option: Option data
        :param value: Option value
        :param subkey: Subkey for specific checkbox
        :param hooks: Run hooks
        """
        # on update hooks
        if hooks:
            hook_name = "update.{}.{}".format(parent_id, key)
            if self.window.ui.has_hook(hook_name):
                hook = self.window.ui.get_hook(hook_name)
                try:
                    hook(key, value, 'bool_list')
                except Exception as e:
                    self.window.core.debug.log(e)

    def get_value(
            self,
            parent_id: str,
            key: str,
            option: Dict[str, Any]
    ) -> str:
        """
        Get checkbox values as comma-separated string

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option data dict
        :return: Option value
        """
        if key not in self.window.ui.config[parent_id]:
            return ""
        imploded_list = []
        for item in self.window.ui.config[parent_id][key].boxes:
            if self.window.ui.config[parent_id][key].boxes[item] is None:
                continue
            if self.window.ui.config[parent_id][key].boxes[item].isChecked():
                imploded_list.append(item)
        return ",".join(imploded_list)


    def update_list(
            self,
            parent_id: str,
            key: str,
            items: List[Dict]
    ):
        """
        Update combobox items

        :param parent_id: Options parent ID
        :param key: Option key
        :param items: Items dict
        """
        self.window.ui.config[parent_id][key].update_boxes_list(items)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.11.26 04:00:00                  #
# ================================================== #

from PySide6.QtCore import Slot, QTimer

from pygpt_net.core.bridge.context import BridgeContext
from pygpt_net.plugin.base.plugin import BasePlugin
from pygpt_net.core.events import Event, KernelEvent
from pygpt_net.item.ctx import CtxItem

from .config import Config
from .worker import Worker


class Plugin(BasePlugin):
    def __init__(self, *args, **kwargs):
        super(Plugin, self).__init__(*args, **kwargs)
        self.id = "cmd_mouse_control"
        self.name = "Mouse And Keyboard"
        self.description = "Provides ability to control mouse and keyboard"
        self.prefix = "Mouse"
        self.order = 100
        self.allowed_cmds = [
            "get_mouse_position",
            "mouse_move",
            "mouse_click",
            "mouse_scroll",
            "get_screenshot",
            "keyboard_key",
            "keyboard_type",
        ]
        self.use_locale = True
        self.worker = None
        self.config = Config(self)
        self.init_options()

    def init_options(self):
        """Initialize options"""
        self.config.from_defaults(self)

    def handle(self, event: Event, *args, **kwargs):
        """
        Handle dispatched event

        :param event: event object
        :param args: event args
        :param kwargs: event kwargs
        """
        name = event.name
        data = event.data
        ctx = event.ctx

        if name == Event.CMD_SYNTAX:
            self.cmd_syntax(data)

        elif name == Event.CMD_EXECUTE:
            self.cmd(
                ctx,
                data['commands'],
            )
        elif name == Event.SYSTEM_PROMPT:
            if self.cmd_exe():
                data['value'] = self.on_system_prompt(data['value'])

    def cmd_syntax(self, data: dict):
        """
        Event: CMD_SYNTAX

        :param data: event data dict
        """
        for option in self.allowed_cmds:
            if self.has_cmd(option):
                data['cmd'].append(self.get_cmd(option))  # append command

    def cmd(self, ctx: CtxItem, cmds: list):
        """
        Event: CMD_EXECUTE

        :param ctx: CtxItem
        :param cmds: commands dict
        """
        is_cmd = False
        my_commands = []
        for item in cmds:
            if item["cmd"] in self.allowed_cmds:
                my_commands.append(item)
                is_cmd = True

        if not is_cmd:
            return

        # set state: busy
        self.cmd_prepare(ctx, my_commands)

        try:
            worker = Worker()
            worker.from_defaults(self)
            worker.cmds = my_commands
            worker.ctx = ctx

            if not self.is_async(ctx):
                worker.run()
                return
            worker.run_async()

        except Exception as e:
            self.error(e)

    @Slot(list, object, dict)
    def handle_finished_more(self, responses: list, ctx: CtxItem = None, extra_data: dict = None):
        """
        Handle finished responses signal

        :param responses: responses list
        :param ctx: context (CtxItem)
        :param extra_data: extra data
        """
        # dispatch response (reply) - collect all responses and make screenshot only once at the end
        for response in responses:
            if ctx is not None:
                ctx.results.append(response)
                ctx.reply = True
        self.handle_delayed(ctx)

    @Slot(object)
    def handle_delayed(self, ctx: CtxItem):
        """
        Handle delayed screenshot

        :param ctx: context (CtxItem)
        """
        if self.get_option_value("allow_screenshot"):
            self.window.controller.attachment.clear_silent()
            path = self.window.controller.painter.capture.screenshot(attach_cursor=True,
                                                                     silent=True)  # attach screenshot

            img_path = self.window.core.filesystem.make_local(path)
            ctx.images.append(img_path)
            # ctx.images_before.append(path)

        context = BridgeContext()
        context.ctx = ctx
        event = KernelEvent(KernelEvent.REPLY_ADD, {
            'context': context,
            'extra': {},
        })
        self.window.dispatch(event)

    def on_system_prompt(self, prompt: str) -> str:
        """
        Event: SYSTEM_PROMPT

        :param prompt: prompt
        :return: updated prompt
        """
        if prompt is not None and prompt.strip() != "":
            prompt += "\n\n"
        return prompt + self.get_option_value("prompt")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.17 21:00:00                  #
# ================================================== #

import json
import os
import shutil
from typing import Optional, Dict, Any

from packaging.version import Version

from pygpt_net.provider.core.config.base import BaseProvider
from .patch import Patch


class JsonFileProvider(BaseProvider):
    def __init__(self, window=None):
        super(JsonFileProvider, self).__init__(window)
        self.window = window
        self.patcher = Patch(window)
        self.id = "json_file"
        self.type = "config"
        self.config_file = 'config.json'
        self.settings_file = 'settings.json'
        self.sections_file = 'settings_section.json'

    def install(self):
        """Install provider data files"""
        # config file
        dst = os.path.join(self.path, self.config_file)
        if not os.path.exists(dst):
            src = os.path.join(self.path_app, 'data', 'config', self.config_file)
            shutil.copyfile(src, dst)
        else:
            # check if config file is correct - if not, then restore from base config
            try:
                with open(dst, 'r', encoding="utf-8") as file:
                    json.load(file)
            except json.JSONDecodeError:
                print("RECOVERY: Config file `{}` is corrupted. Restoring from base config.".format(dst))
                backup_dst = os.path.join(self.path, 'config.bak.json')
                if os.path.exists(backup_dst):
                    os.remove(backup_dst)
                shutil.copyfile(dst, backup_dst)
                os.remove(dst)
                print("RECOVERY: Backup of corrupted config file created: {}".format(backup_dst))
                src = os.path.join(self.path_app, 'data', 'config', self.config_file)
                shutil.copyfile(src, dst)
                print("RECOVERY: Restored config file from base config: {}".format(src))

        # data/tmp directory
        tmp_dir = os.path.join(self.window.core.config.get_user_dir('data'), 'tmp')
        if not os.path.exists(tmp_dir):
            try:
                os.makedirs(tmp_dir, exist_ok=True)
            except Exception as e:
                print("WARNING: Cannot create ``{}`` temp directory: {}".format(tmp_dir, e))

    def get_version(self) -> Optional[str]:
        """
        Get config file version

        :return: version
        """
        path = os.path.join(self.path, self.config_file)
        with open(path, 'r', encoding="utf-8") as file:
            data = json.load(file)
            if data == "" or data is None:
                return
            if '__meta__' in data and 'version' in data['__meta__']:
                return data['__meta__']['version']

    def load(self, all: bool = False) -> Optional[Dict[str, Any]]:
        """
        Load config from JSON file

        :param all: if True, print loaded config message
        :return: dict with data or None if file not found
        """
        data = {}
        path = os.path.join(self.path, self.config_file)
        if not os.path.exists(path):
            print("User config: {} not found.".format(path))
            return None
        try:
            with open(path, 'r', encoding="utf-8") as f:
                data = json.load(f)
                if all:
                    print("Loaded config: {}".format(path))
        except Exception as e:
            print("FATAL ERROR: {}".format(e))
        return data

    def load_base(self) -> Optional[Dict[str, Any]]:
        """
        Load config from JSON file

        :return: dict with data or None if file not found
        """
        data = {}
        path = os.path.join(self.path_app, 'data', 'config', self.config_file)
        if not os.path.exists(path):
            print("FATAL ERROR: {} not found!".format(path))
            return None
        try:
            with open(path, 'r', encoding="utf-8") as f:
                data = json.load(f)
        except Exception as e:
            print("FATAL ERROR: {}".format(e))
        return data

    def save(self, data: Dict[str, Any], filename: str = 'config.json'):
        """
        Save config to JSON file

        :param dict with data: data to save
        :param filename: filename, default: config.json
        """
        path = os.path.join(self.path, filename)
        try:
            data['__meta__'] = self.meta
            dump = json.dumps(data, indent=4)
            with open(path, 'w', encoding="utf-8") as f:
                f.write(dump)
        except Exception as e:
            print("FATAL ERROR: {}".format(e))

    def get_options(self) -> Optional[Dict[str, Any]]:
        """
        Load config settings options from JSON file

        :return: dict with data or None if file not found
        """
        data = {}
        path = os.path.join(self.path_app, 'data', 'config', self.settings_file)
        if not os.path.exists(path):
            print("FATAL ERROR: {} not found!".format(path))
            return None
        try:
            with open(path, 'r', encoding="utf-8") as f:
                data = json.load(f)
        except Exception as e:
            print("FATAL ERROR: {}".format(e))
        return data

    def get_sections(self) -> Optional[Dict[str, Any]]:
        """
        Load config sections from JSON file

        :return: dict with data or None if file not found
        """
        data = {}
        path = os.path.join(self.path_app, 'data', 'config', self.sections_file)
        if not os.path.exists(path):
            print("FATAL ERROR: {} not found!".format(path))
            return None
        try:
            with open(path, 'r', encoding="utf-8") as f:
                data = json.load(f)
        except Exception as e:
            print("FATAL ERROR: {}".format(e))
        return data

    def patch(self, version: Version) -> bool:
        """
        Migrate config to current app version

        :param version: current app version
        :return: True if migrated
        """
        return self.patcher.execute(version)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.24 02:00:00                  #
# ================================================== #
from PySide6 import QtCore
from PySide6.QtWidgets import QPushButton, QHBoxLayout, QVBoxLayout, QLabel

from pygpt_net.ui.widget.lists.model_importer import ModelImporter
from pygpt_net.ui.widget.dialog.model_importer import ModelImporterOllamaDialog
from pygpt_net.utils import trans


class ModelsImporter:
    def __init__(self, window=None):
        """
        Models importer dialog

        :param window: Window instance
        """
        self.window = window
        self.dialog_id = "models.importer"

    def setup(self, idx=None):
        """
        Setup dialog

        :param idx: current model tab index
        """
        # models
        self.window.ui.nodes['models.importer.editor'] = ModelImporter(self.window)

        self.window.ui.nodes['models.importer.btn.refresh'] = \
            QPushButton(trans("dialog.models.importer.btn.refresh"))
        self.window.ui.nodes['models.importer.btn.cancel'] = \
            QPushButton(trans("dialog.models.importer.btn.cancel"))
        self.window.ui.nodes['models.importer.btn.save'] = \
            QPushButton(trans("dialog.models.importer.btn.save"))

        self.window.ui.nodes['models.importer.btn.refresh'].clicked.connect(
            lambda: self.window.controller.model.importer.refresh(reload=True))
        self.window.ui.nodes['models.importer.btn.cancel'].clicked.connect(
            lambda: self.window.controller.model.importer.cancel())
        self.window.ui.nodes['models.importer.btn.save'].clicked.connect(
            lambda: self.window.controller.model.importer.save())

        # set enter key to save button
        self.window.ui.nodes['models.importer.btn.refresh'].setAutoDefault(False)
        self.window.ui.nodes['models.importer.btn.cancel'].setAutoDefault(False)
        self.window.ui.nodes['models.importer.btn.save'].setAutoDefault(True)

        # footer buttons
        footer = QHBoxLayout()
        footer.addWidget(self.window.ui.nodes['models.importer.btn.refresh'])
        footer.addWidget(self.window.ui.nodes['models.importer.btn.cancel'])
        footer.addWidget(self.window.ui.nodes['models.importer.btn.save'])

        self.window.ui.nodes["models.importer.url"] = QLabel("S")
        self.window.ui.nodes["models.importer.url"].setAlignment(QtCore.Qt.AlignRight)
        self.window.ui.nodes["models.importer.url"].setContentsMargins(10, 10, 10, 10)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.window.ui.nodes["models.importer.url"])
        main_layout.addWidget(self.window.ui.nodes['models.importer.editor'])

        self.window.ui.nodes["models.importer.status"] = QLabel("S")
        self.window.ui.nodes["models.importer.status"].setAlignment(QtCore.Qt.AlignCenter)
        self.window.ui.nodes["models.importer.status"].setContentsMargins(10, 10, 10, 10)

        layout = QVBoxLayout()
        layout.addLayout(main_layout)  # list
        layout.addWidget(self.window.ui.nodes["models.importer.status"])
        layout.addLayout(footer)  # bottom buttons (save, defaults)

        self.window.ui.dialog[self.dialog_id] = ModelImporterOllamaDialog(self.window, self.dialog_id)
        self.window.ui.dialog[self.dialog_id].setLayout(layout)
        self.window.ui.dialog[self.dialog_id].setWindowTitle(trans('dialog.models.importer'))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.13 01:00:00                  #
# ================================================== #

from typing import Dict, Any

# from llama_index.core.agent.workflow import CodeActAgent as Agent
from .codeact_agent_custom import DEFAULT_CODE_ACT_PROMPT, CodeActAgent as Agent  # <-- custom version with tools

from pygpt_net.core.types import (
    AGENT_MODE_WORKFLOW,
)
from .base import BaseAgent

class CodeActAgent(BaseAgent):

    def __init__(self, *args, **kwargs):
        super(CodeActAgent, self).__init__(*args, **kwargs)
        self.id = "code_act"
        self.mode = AGENT_MODE_WORKFLOW
        self.name = "CodeAct (Workflow)"

    def get_agent(self, window, kwargs: Dict[str, Any]):
        """
        Return Agent provider instance

        :param window: window instance
        :param kwargs: keyword arguments
        :return: Agent provider instance
        """
        tools = kwargs.get("plugin_tools", {})
        specs = kwargs.get("plugin_specs", [])
        retriever_tool = kwargs.get("retriever_tools", None)
        workdir = kwargs.get("workdir", "/data")
        llm = kwargs.get("llm", None)
        system_prompt = kwargs.get("system_prompt", "")
        are_cmds = kwargs.get("are_commands", True)
        kwargs = {
            "code_execute_fn": window.core.agents.tools.code_execute_fn.execute,
            "plugin_tool_fn": window.core.agents.tools.tool_exec,
            "plugin_tools": tools,
            "plugin_specs": specs,
            "tool_retriever": retriever_tool,
            "llm": llm,
            "system_prompt": system_prompt,
            "code_act_system_prompt": DEFAULT_CODE_ACT_PROMPT.replace("{workdir}", workdir),
        }
        return Agent(**kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.11.18 19:00:00                  #
# ================================================== #

from pygpt_net.plugin.base.config import BaseConfig, BasePlugin


class Config(BaseConfig):
    def __init__(self, plugin: BasePlugin = None, *args, **kwargs):
        super(Config, self).__init__(plugin)
        self.plugin = plugin

    def from_defaults(self, plugin: BasePlugin = None):
        """
        Set default options for plugin

        :param plugin: plugin instance
        """
        plugin.add_option(
            "provider",
            type="combo",
            value="openai_tts",
            label="Provider",
            description="Select audio output provider, default: OpenAI TTS",
            tooltip="Select audio output provider",
            keys=plugin.get_provider_options(),
        )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.18 00:00:00                  #
# ================================================== #

import copy
import os
from pathlib import Path
from typing import Optional
from uuid import uuid4

from PySide6.QtCore import QObject, QRunnable, Signal, Slot

from pygpt_net.utils import trans


class WorkerSignals(QObject):
    """
    Signals for worker to communicate with main thread.
    """
    finished = Signal()  # when worker is finished
    deleted = Signal(str)  # after deleting profile
    duplicated = Signal(str, str)  # after duplicating profile (str: new_uuid, str: new_name)
    error = Signal(str)  # when error occurs
    hideStatus = Signal()  # hide status in dialog
    updateStatus = Signal(str)    # update status in dialog
    updateGlobalStatus = Signal(str)  # update global status
    alert = Signal(object)        # dialog alert
    reload = Signal()             # on reload components (e.g. after workdir change)
    confirm = Signal(str, str, str)  # confirm dialog (action, path, message)
    restored = Signal(str)        # after restoring workdir
    updated = Signal(str)         # after updating workdir
    switch = Signal(str)       # switch to profile (after reset)


class WorkdirWorker(QObject, QRunnable):
    """Worker for handling workdir operations in a separate thread."""
    def __init__(
            self,
            window,
            action: str,
            path: str = None,
            force: bool = False,
            current: Optional[str] = None,
            profile_name: Optional[str] = None,
            profile_uuid: Optional[str] = None,
            profile_new_name: Optional[str] = None,
            profile_new_path: Optional[str] = None,
    ):
        QObject.__init__(self)
        QRunnable.__init__(self)
        self.window = window
        self.action = action
        self.path = path
        self.force = force
        self.current = current
        self.profile_name = profile_name
        self.profile_uuid = profile_uuid
        self.profile_new_name = profile_new_name
        self.profile_new_path = profile_new_path
        self.signals = WorkerSignals()

    @Slot()
    def run(self):
        try:
            if self.action == "update":
                self.worker_update()
            elif self.action == "migrate":
                self.worker_migrate()
            elif self.action == "restore":
                self.worker_restore()
            elif self.action == "delete":
                self.worker_delete_files()
            elif self.action == "duplicate":
                self.worker_duplicate()
            elif self.action == "reset":
                self.worker_reset()
            else:
                self.signals.error.emit(f"Unknown action: {self.action}")
        except Exception as e:
            self.window.core.debug.log(e)
            self.signals.error.emit(str(e))
        finally:
            self.signals.finished.emit()

    def worker_delete_files(self):
        """Delete files and directories associated with the profile"""
        uuid = self.profile_uuid
        profiles = self.window.controller.settings.profile.get_profiles()
        remove_datadir = True
        remove_db = True
        if uuid in profiles:
            profile = profiles[uuid]
            name = profile['name']
            path = profile['workdir'].replace("%HOME%", str(Path.home()))
            # remove profile
            if self.window.core.config.profile.remove(uuid):
                if not os.path.exists(path) or not os.path.isdir(path):
                    self.signals.alert.emit(trans("dialog.profile.alert.path.not_exists"))
                    return
                print("Clearing workdir: ", path)
                self.window.core.filesystem.clear_workdir(
                    path,
                    remove_db=remove_db,
                    remove_datadir=remove_datadir,
                )
                self.signals.deleted.emit(name)

    def worker_duplicate(self):
        """Duplicate profile"""
        uuid = self.profile_uuid
        new_name = self.profile_new_name
        new_path = self.profile_new_path

        profiles = self.window.controller.settings.profile.get_profiles()
        if uuid not in profiles:
            self.signals.alert.emit("Profile not found!")
            return
        profile = profiles[uuid]

        copy_datadir = self.window.controller.settings.profile.is_include_datadir()
        copy_db = self.window.controller.settings.profile.is_include_db()

        # make copy
        duplicate = copy.deepcopy(profile)
        new_uuid = str(uuid4())
        duplicate['name'] = new_name
        duplicate['workdir'] = new_path
        self.window.core.config.profile.append(new_uuid, duplicate)

        # copy files from workdir
        path_from = profile['workdir'].replace("%HOME%", str(Path.home()))
        path_to = new_path
        print("Copying all files from {} to: {}".format(path_from, path_to))
        self.signals.updateGlobalStatus.emit("Copying files...")
        result = self.window.core.filesystem.copy_workdir(
            path_from,
            path_to,
            copy_db=copy_db,
            copy_datadir=copy_datadir,
        )
        if not result:
            self.signals.alert.emit("Error copying files!")
            self.signals.updateGlobalStatus.emit("Error copying files!")
            return
        print("[OK] All files copied successfully.")
        self.signals.duplicated.emit(new_uuid, new_name)

    def worker_reset(self):
        """Reset profile"""
        uuid = self.profile_uuid
        profiles = self.window.controller.settings.profile.get_profiles()
        current = self.window.core.config.profile.get_current()
        remove_datadir = False
        remove_db = True
        if uuid in profiles:
            profile = profiles[uuid]
            path = profile['workdir'].replace("%HOME%", str(Path.home()))
            if not os.path.exists(path) or not os.path.isdir(path):
                self.signals.alert.emit("Directory not exists!")
                return
            print("Clearing workdir: ", path)
            self.window.core.db.close()
            self.window.core.filesystem.clear_workdir(
                path,
                remove_db=remove_db,
                remove_datadir=remove_datadir,
            )
            if uuid == current:
                self.signals.switch.emit(uuid)  # switch to profile
            self.signals.updateGlobalStatus.emit("Profile cleared: " + profile['name'])

    def worker_update(self):
        """Switch working directory to the existing one"""
        print("\n====================")
        print("Changing workdir to: ", self.path)
        print("====================\n")
        current_path = self.window.core.config.get_user_path()
        default_path = self.window.core.config.get_base_workdir()

        if self.force:
            self.signals.updateStatus.emit(trans("dialog.workdir.result.wait"))

        lock_file = os.path.join(default_path, 'path.cfg')  # put "path.cfg"
        lock_path = self.path.replace(str(Path.home()), "%HOME%")
        if self.path == default_path:
            lock_path = ""  # set empty if default dir
        with open(lock_file, 'w', encoding='utf-8') as f:
            f.write(lock_path)

         # update path in current profile
        self.window.core.config.profile.update_current_workdir(self.path)

        # reload config
        self.window.core.config.set_workdir(self.path, reload=True)
        self.window.core.config.set('license.accepted', True)  # accept license to prevent show dialog again

        # reload components
        if self.force:
            try:
                self.signals.reload.emit()
                success_msg = trans("dialog.workdir.result.success").format(path=self.path)
                self.signals.updateStatus.emit(success_msg)
                self.signals.alert.emit(success_msg)
            except Exception as e:
                self.window.core.debug.log(e)
                self.signals.alert.emit(str(e))
                print("Error reloading components: ", e)
                self.worker_restore(custom_current=current_path)
                self.window.controller.reloading = False  # unlock
        else:
            # always reload
            self.signals.reload.emit()

        # update profile after workdir change
        if self.profile_name:
            self.signals.updated.emit(self.profile_name)

    def worker_migrate(self):
        """Migrate working directory"""
        if self.window.controller.settings.workdir.busy:
            self.signals.alert.emit("Workdir migration in progress...")
            return

        # check if not the same directory
        current = self.window.core.config.get_user_path()
        if current == self.path:
            self.signals.alert.emit(trans("dialog.workdir.result.same_directory"))
            self.window.controller.settings.workdir.busy = False
            self.signals.hideStatus.emit()
            return

        # check if path is not already a workdir
        if self.window.core.filesystem.is_workdir_in_path(self.path):
            msg = trans("dialog.workdir.update.confirm").format(path=self.path)
            self.signals.confirm.emit("workdir.update", self.path, msg)
            return

        # check if path is empty
        if not self.window.core.filesystem.is_directory_empty(self.path):
            self.signals.alert.emit(trans("dialog.workdir.change.empty.alert"))
            return

        # confirm move
        if not self.force:
            msg = trans("dialog.workdir.change.confirm").format(path=self.path)
            self.signals.confirm.emit("workdir.change", self.path, msg)
            return

        self.signals.hideStatus.emit()
        self.window.controller.settings.workdir.busy = True
        print("Migrating workdir from: ", current, " to: ", self.path)

        # check if path exists
        if not os.path.exists(self.path) or not os.path.isdir(self.path):
            self.signals.alert.emit(trans("dialog.workdir.result.directory_not_exists"))
            self.signals.hideStatus.emit()
            self.window.controller.settings.workdir.busy = False
            return

        # check free space
        space_required = self.window.core.filesystem.get_directory_size(current, human_readable=False)
        space_free = self.window.core.filesystem.get_free_disk_space(self.path, human_readable=False)
        if space_required > space_free:
            self.signals.alert.emit(trans("dialog.workdir.result.no_free_space").format(
                required=self.window.core.filesystem.get_directory_size(current),
                free=self.window.core.filesystem.get_free_disk_space(self.path),
            ))
            self.window.controller.settings.workdir.busy = False
            self.signals.hideStatus.emit()
            return

        # copy workdir
        try:
            result = self.window.core.filesystem.copy_workdir(current, self.path)
        except Exception as e:
            self.window.core.debug.log(e)
            self.signals.alert.emit(str(e))
            print("Error migrating workdir: ", e)
            result = False

        if result:
            try:
                # remove old workdir
                self.window.core.debug.info("Clearing old workdir: {}".format(current))
                try:
                    # allow errors here
                    self.window.core.filesystem.clear_workdir(current)
                except Exception as e:
                    self.window.core.debug.log(e)
                    print("Error clearing old workdir: ", e)

                  # update workdir to new path
                self.worker_update()
                success_msg = trans("dialog.workdir.result.success").format(path=self.path)
                print(success_msg)
                self.signals.updateStatus.emit(success_msg)
                self.signals.alert.emit(success_msg)
            except Exception as e:
                self.window.core.debug.log(e)
                self.signals.alert.emit(str(e))
                print("Error migrating workdir: ", e)
                self.worker_restore(custom_current=current)
                self.window.controller.reloading = False
        else:
            self.signals.updateStatus.emit(trans("dialog.workdir.result.failed"))
            self.signals.alert.emit(trans("dialog.workdir.result.failed"))
            self.worker_restore(custom_current=current)
            self.window.controller.reloading = False

        self.window.controller.settings.workdir.busy = False
        self.window.core.debug.info("Finished migrating workdir from: {} to: {}".format(current, self.path))

    def worker_restore(self, custom_current: str = None):
        """
        Restore default working directory

        :param custom_current: custom current working directory (optional)
        """
        current = custom_current if custom_current is not None else self.current
        print("Reverting workdir to: ", current)
        self.window.core.config.set_workdir(current, reload=True)
        default_path = self.window.core.config.get_base_workdir()
        lock_file = os.path.join(default_path, 'path.cfg')
        lock_path = current.replace(str(Path.home()), "%HOME%")
        if current == default_path:
            lock_path = ""
        with open(lock_file, 'w', encoding='utf-8') as f:
            f.write(lock_path)
        self.signals.restored.emit(current)
        self.signals.updateStatus.emit("Failed. Reverted to current workdir: {}".format(current))
        self.signals.reload.emit()
        self.window.core.config.profile.update_current_workdir(current)


class Workdir:
    def __init__(self, window=None):
        """
        Workdir controller

        :param window: window instance
        """
        self.window = window
        self.is_dialog = False
        self.busy = False
        self.worker = None

    def change(self):
        """Change working directory (open dialog)"""
        if self.is_dialog:
            self.window.ui.dialogs.close('workdir.change')
            self.is_dialog = False
        else:
            self.window.ui.dialogs.workdir.prepare()
            self.window.ui.dialogs.workdir.set_path(self.window.core.config.get_user_path())
            self.window.ui.dialogs.workdir.hide_status()
            self.window.ui.dialogs.open(
                'workdir.change',
                width=600,
                height=180,
            )
            self.is_dialog = True

    def run_action(
            self,
            action: str,
            path: str = None,
            force: bool = False,
            current: str = None,
            profile_name: Optional[str] = None,
            profile_uuid: Optional[str] = None,
            profile_new_name: Optional[str] = None,
            profile_new_path: Optional[str] = None,
    ):
        """
        Run action in thread

        :param action: action to perform (update, migrate, restore)
        :param path: path for update or migrate
        :param force: force action (confirm)
        :param current: current working directory (for restore action)
        :param profile_name: profile name (optional, for after update callback)
        :param profile_uuid: profile UUID (optional, for delete files action)
        :param profile_new_name: new profile name (optional, for duplicate action)
        :param profile_new_path: new profile path (optional, for duplicate action)
        """
        self.worker = WorkdirWorker(
            window=self.window,
            action=action,
            path=path,
            force=force,
            current=current,
            profile_name=profile_name,
            profile_uuid=profile_uuid,
            profile_new_name=profile_new_name,
            profile_new_path=profile_new_path,
        )

        # connect signals
        self.worker.signals.updateGlobalStatus.connect(self.window.update_status)
        self.worker.signals.updateStatus.connect(self.window.ui.dialogs.workdir.show_status)
        self.worker.signals.hideStatus.connect(self.window.ui.dialogs.workdir.hide_status)
        self.worker.signals.alert.connect(self.window.ui.dialogs.alert)
        self.worker.signals.reload.connect(self.window.controller.reload)
        self.worker.signals.error.connect(lambda err: self.window.core.debug.log(f"Worker error: {err}"))
        self.worker.signals.confirm.connect(self.window.ui.dialogs.confirm)
        self.worker.signals.restored.connect(lambda current: self.window.ui.dialogs.workdir.set_path(current))
        self.worker.signals.updated.connect(self.window.controller.settings.profile.after_update)
        self.worker.signals.deleted.connect(self.window.controller.settings.profile.after_delete)
        self.worker.signals.duplicated.connect(self.window.controller.settings.profile.after_duplicate)
        self.worker.signals.switch.connect(self.window.controller.settings.profile.switch_current)

        # start worker in thread pool
        self.window.threadpool.start(self.worker)

    def update(
            self,
            path: str,
            force: bool = False,
            profile_name: str = None
    ):
        """
        Switch working directory to the existing one

        :param path: existing working directory
        :param force: force update (confirm)
        :param profile_name: profile name (optional, for future use)
        """
        self.run_action(
            action="update",
            path=path,
            force=force,
            profile_name=profile_name,
        )

    def migrate(
            self,
            path: str,
            force: bool = False
    ):
        """
        Migrate working directory

        :param path: new working directory
        :param force: force migration (confirm)
        """
        self.run_action(
            action="migrate",
            path=path,
            force=force,
        )

    def restore(
            self,
            current: str
    ):
        """
        Restore default working directory

        :param current: current working directory
        """
        self.run_action(
            action="restore",
            current=current,
        )

    def delete_files(
            self,
            profile_uuid: str
    ):
        """
        Delete files and directories associated with the profile

        :param profile_uuid: profile UUID
        """
        self.run_action(
            action="delete",
            profile_uuid=profile_uuid,
        )

    def duplicate(
            self,
            profile_uuid: str,
            new_name: str,
            new_path: str
    ):
        """
        Duplicate profile

        :param profile_uuid: profile UUID to duplicate
        :param new_name: new profile name
        :param new_path: new profile path
        """
        self.run_action(
            action="duplicate",
            profile_uuid=profile_uuid,
            profile_new_name=new_name,
            profile_new_path=new_path,
        )

    def reset(
            self,
            profile_uuid: str
    ):
        """
        Reset profile

        :param profile_uuid: profile UUID to reset
        """
        self.run_action(
            action="reset",
            profile_uuid=profile_uuid,
        )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.07.19 00:00:00                  #
# ================================================== #

import copy
from typing import Optional, List, Dict

from httpx_socks import SyncProxyTransport
from openai import DefaultHttpxClient
from packaging.version import Version

from pygpt_net.core.types import (
    MODE_CHAT,
    MODE_LANGCHAIN,
    MODE_LLAMA_INDEX,
    MODE_RESEARCH,
    MULTIMODAL_TEXT,
    MULTIMODAL_IMAGE,
    MULTIMODAL_AUDIO,
    MULTIMODAL_VIDEO,
    MODEL_DEFAULT_MINI,
)
from pygpt_net.item.model import ModelItem
from pygpt_net.provider.core.model.json_file import JsonFileProvider

from .ollama import Ollama

class Models:
    def __init__(self, window=None):
        """
        Models core

        :param window: Window instance
        """
        self.window = window
        self.provider = JsonFileProvider(window)
        self.ollama = Ollama(window)
        self.default = MODEL_DEFAULT_MINI
        self.items = {}
        self.multimodal = [
            MULTIMODAL_TEXT,
            MULTIMODAL_IMAGE,
            MULTIMODAL_AUDIO,
            MULTIMODAL_VIDEO,
        ]

    def install(self):
        """Install provider data"""
        self.provider.install()

    def patch(self, app_version: Version) -> bool:
        """
        Patch provider data

        :param app_version: app version
        :return: True if data was patched
        """
        return self.provider.patch(app_version)

    def patch_missing(self) -> bool:
        """
        Patch missing models

        :return: True if models were patched
        """
        base_items = self.get_base()
        updated = False
        added_keys = []

        # check for missing keys
        for key in base_items:
            if key not in self.items:
                self.items[key] = copy.deepcopy(base_items[key])
                updated = True

        # check for missing models ids
        for key in base_items:
            model_id = base_items[key].id
            old_exists = False
            for old_key in self.items:
                if self.items[old_key].id == model_id:
                    old_exists = True
                    break
            if not old_exists and key not in added_keys:
                self.items[key] = copy.deepcopy(base_items[key])
                added_keys.append(key)
                updated = True

        # update multimodal options
        for key in self.items:
            if key in base_items:
                if base_items[key].input != self.items[key].input:
                    self.items[key].input = base_items[key].input
                    updated = True
                if base_items[key].output != self.items[key].output:
                    self.items[key].output = base_items[key].output
                    updated = True

        # update empty multimodal options
        for key in self.items:
            if isinstance(self.items[key].input, list):
                if len(self.items[key].input) == 0:
                    self.items[key].input = ["text"]
                    updated = True
            if isinstance(self.items[key].output, list):
                if len(self.items[key].output) == 0:
                    self.items[key].output = ["text"]
                    updated = True

        if updated:
            self.save()

        return updated

    def get(self, key: str) -> ModelItem:
        """
        Return model config

        :param key: model name
        :return: model config object
        """
        if key in self.items:
            return self.items[key]

    def get_ids(self) -> List[str]:
        """
        Return models ids

        :return: model ids list
        """
        return list(self.items.keys())

    def get_id_by_idx_all(self, idx: int) -> str:
        """
        Return model id by index

        :param idx: model idx
        :return: model id
        """
        return list(self.items.keys())[idx]

    def has(self, model: str) -> bool:
        """
        Check if model exists

        :param model: model name
        :return: True if model exists
        """
        return model in self.items

    def is_allowed(
            self,
            model: str,
            mode: str
    ) -> bool:
        """
        Check if model is allowed for mode

        :param model: model name
        :param mode: mode name
        :return: True if model is allowed for mode
        """
        if model in self.items:
            return mode in self.items[model].mode
        return False

    def get_id(
            self,
            key: str
    ) -> str:
        """
        Return model internal ID

        :param key: model key
        :return: model id
        """
        if key in self.items:
            return self.items[key].id

    def get_by_idx(
            self,
            idx: int,
            mode: str
    ) -> str:
        """
        Return model by index

        :param idx: model idx
        :param mode: mode name
        :return: model name
        """
        items = self.get_by_mode(mode)
        return list(items.keys())[idx]

    def get_by_mode(
            self,
            mode: str
    ) -> Dict[str, ModelItem]:
        """
        Return models for mode

        :param mode: mode name
        :return: models dict for mode
        """
        items = {}
        for key in self.items:
            if mode in self.items[key].mode:
                items[key] = self.items[key]
        return items

    def get_next(
            self,
            model: str,
            mode: str
    ) -> str:
        """
        Return next model

        :param model: current model
        :param mode: mode name
        :return: next model
        """
        items = self.get_by_mode(mode)
        keys = list(items.keys())
        idx = keys.index(model)
        if idx + 1 < len(keys):
            return keys[idx + 1]
        return keys[0]

    def get_prev(
            self,
            model: str,
            mode: str
    ) -> str:
        """
        Return previous model

        :param model: current model
        :param mode: mode name
        :return: previous model
        """
        items = self.get_by_mode(mode)
        keys = list(items.keys())
        idx = keys.index(model)
        if idx - 1 >= 0:
            return keys[idx - 1]
        return keys[-1]

    def create_id(self):
        """
        Create new model id

        :return: new model id
        """
        id = "model-000"
        while id in self.items:
            id = "model-" + str(int(id.split("-")[1]) + 1).zfill(3)
        return id

    def get_multimodal_list(self) -> List[str]:
        """
        Return available multimodal types

        :return: list of multimodal types
        """
        return self.multimodal

    def create_empty(self, append: bool = True) -> ModelItem:
        """
        Create new empty model

        :param append: if True, append model to items
        :return: new model

        """
        id = self.create_id()
        model = ModelItem()
        model.id = id
        model.name = "New model"
        if append:
            self.items[id] = model
        return model

    def get_all(self) -> Dict[str, ModelItem]:
        """
        Return all models

        :return: all models
        """
        return self.items

    def from_defaults(self) -> ModelItem:
        """
        Create default model

        :return: new model
        """
        model = ModelItem()
        model.id = self.default
        model.name = self.default
        model.tokens = 4096
        model.ctx = 128000
        model.input = ["text"]
        model.output = ["text"]
        model.provider = "openai"
        model.mode = ["chat"]
        return model

    def delete(self, model: str):
        """
        Delete model

        :param model: model name
        """
        if model in self.items:
            del self.items[model]

    def has_model(
            self,
            mode: str,
            model: str
    ) -> bool:
        """
        Check if model exists for mode

        :param mode: mode name
        :param model: model name
        :return: True if model exists for mode
        """
        items = self.get_by_mode(mode)
        return model in items

    def get_default(self, mode: str) -> Optional[str]:
        """
        Return default model for mode

        :param mode: mode name
        :return: default model name
        """
        models = {}
        items = self.get_by_mode(mode)
        for k in items:
            models[k] = items[k]
        if len(models) == 0:
            return None
        return list(models.keys())[0]

    def get_tokens(self, model: str) -> int:
        """
        Return model tokens

        :param model: model name
        :return: number of tokens
        """
        if model in self.items:
            return self.items[model].tokens
        return 1

    def get_num_ctx(self, model: str) -> int:
        """
        Return model context window tokens

        :param model: model name
        :return: number of ctx tokens
        """
        if model in self.items:
            return self.items[model].ctx
        return 4096

    def restore_default(
            self,
            model: Optional[str] = None
    ):
        """
        Restore default models

        :param model: model name
        """
        # restore all models
        if model is None:
            self.load_base()
            return

        # restore single model
        items = self.provider.load_base()
        if model in items:
            self.items[model] = items[model]

    def get_base(self) -> Dict[str, ModelItem]:
        """
        Get base models

        :return: base models
        """
        return self.provider.load_base()

    def load_base(self):
        """Load models base"""
        self.items = self.get_base()
        self.sort_items()

    def load(self):
        """Load models"""
        self.items = self.provider.load()
        self.sort_items()

    def sort_items(self):
        """Sort items"""
        if self.items:
            self.items = dict(sorted(self.items.items(), key=lambda x: x[1].name.lower()))

    def save(self):
        """Save models"""
        self.provider.save(self.items)

    def get_supported_mode(
            self,
            model: ModelItem,
            mode: str
    ) -> str:
        """
        Get supported mode

        :param model: ModelItem
        :param mode: mode (initial)
        :return: mode (supported)
        """
        prev_mode = mode
        # if OpenAI API model and not llama_index mode, switch to Chat mode
        if model.is_supported(MODE_CHAT) and mode != MODE_LLAMA_INDEX:  # do not switch if llama_index mode!
            if prev_mode != MODE_CHAT:
                self.window.core.debug.info(
                    "WARNING: Switching to chat mode (model not supported in: {})".format(prev_mode))
            return MODE_CHAT

        # Research / Perplexity
        if model.is_supported(MODE_RESEARCH):
            if prev_mode != MODE_RESEARCH:
                self.window.core.debug.info(
                    "WARNING: Switching to research mode (model not supported in: {})".format(mode))
            mode = MODE_RESEARCH

        # Llama Index / Chat with Files
        elif model.is_supported(MODE_LLAMA_INDEX):
            if prev_mode != MODE_LLAMA_INDEX:
                self.window.core.debug.info(
                    "WARNING: Switching to llama_index mode (model not supported in: {})".format(mode))
            mode = MODE_LLAMA_INDEX

        # LangChain
        """
        elif model.is_supported(MODE_LANGCHAIN):
            self.window.core.debug.info(
                "WARNING: Switching to langchain mode (model not supported in: {})".format(mode))
            mode = MODE_LANGCHAIN
        """
        return mode

    def prepare_client_args(
            self,
            mode: str = MODE_CHAT,
            model: ModelItem = None
    ) -> Dict[str, str]:
        """
        Prepare chat client arguments

        :param mode: mode name
        :param model: ModelItem
        :return: client arguments dict
        """
        args = {
            "api_key": self.window.core.config.get('api_key'),
            "organization": self.window.core.config.get('organization_key'),
        }
        # api endpoint
        if self.window.core.config.has('api_endpoint'):
            endpoint = self.window.core.config.get('api_endpoint')
            if endpoint:
                args["base_url"] = endpoint
        # proxy
        if self.window.core.config.has('api_proxy'):
            proxy = self.window.core.config.get('api_proxy')
            if proxy:
                transport = SyncProxyTransport.from_url(proxy)
                args["http_client"] = DefaultHttpxClient(
                    transport=transport,
                )

        # research mode endpoint - Perplexity
        if model is not None:
            # xAI / grok
            if model.provider == "x_ai":
                args["api_key"] = self.window.core.config.get('api_key_xai', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_xai', "")
                self.window.core.debug.info("[api] Using client: xAI")
            # Perplexity
            elif model.provider == "perplexity":
                args["api_key"] = self.window.core.config.get('api_key_perplexity', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_perplexity', "")
                self.window.core.debug.info("[api] Using client: Perplexity")
            # Google
            elif model.provider == "google":
                args["api_key"] = self.window.core.config.get('api_key_google', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_google', "")
                self.window.core.debug.info("[api] Using client: Google")
            # Anthropic
            elif model.provider == "anthropic":
                args["api_key"] = self.window.core.config.get('api_key_anthropic', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_anthropic', "")
                self.window.core.debug.info("[api] Using client: Anthropic")
            # Deepseek
            elif model.provider == "deepseek_api":
                args["api_key"] = self.window.core.config.get('api_key_deepseek', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_deepseek', "")
                self.window.core.debug.info("[api] Using client: Deepseek API")
            # Mistral AI
            elif model.provider == "mistral_ai":
                args["api_key"] = self.window.core.config.get('api_key_mistral', "")
                args["base_url"] = self.window.core.config.get('api_endpoint_mistral', "")
                self.window.core.debug.info("[api] Using client: Mistral AI API")
            else:
                self.window.core.debug.info("[api] Using client: OpenAI (default)")

            # do not include organization for non-OpenAI providers
            if model.provider != "openai":
                if "organization" in args:
                    del args["organization"]
        else:
            self.window.core.debug.info("[api] No model provided, using default OpenAI client")
        return args

    def is_tool_call_allowed(self, mode: str, model: ModelItem) -> bool:
        """
        Check if native tool call is allowed for model and mode

        :param mode: Mode name
        :param model: ModelItem
        :return: True if tool call is allowed, False otherwise
        """
        if mode == MODE_LLAMA_INDEX:
            idx = self.window.core.config.get('llama.idx.current')
            cmd_enabled = self.window.core.config.get("cmd", False)  # use tools
            use_react = self.window.core.config.get("llama.idx.react", False)  # use ReAct agent for tool calls
            if idx is not None and idx != "_" and cmd_enabled and not use_react:
                return False # do not use native tool calls in Llama Index mode if ReAct agent is not used and idx is set
            if model.provider == "google":
                stream = self.window.core.config.get('stream', False)
                use_react = self.window.core.config.get("llama.idx.react", False)  # use ReAct agent for tool calls
                if stream:
                    if use_react:
                        return True
                    else:
                        return False
        if model.tool_calls:
            return True
        return False

    def get_version(self) -> str:
        """
        Get config version

        :return: config version
        """
        return self.provider.get_version()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.06.30 02:00:00                  #
# ================================================== #

from typing import Any, Dict, List

from .field.checkbox import Checkbox
from .field.checkbox_list import CheckboxList
from .field.combo import Combo
from .field.cmd import Cmd
from .field.dictionary import Dictionary
from .field.input import Input
from .field.slider import Slider
from .field.textarea import Textarea
from .placeholder import Placeholder


class Config:
    def __init__(self, window=None):
        """
        Configuration controller

        :param window: Window instance
        """
        self.window = window
        self.checkbox = Checkbox(window)
        self.checkbox_list = CheckboxList(window)
        self.combo = Combo(window)
        self.cmd = Cmd(window)
        self.dictionary = Dictionary(window)
        self.input = Input(window)
        self.placeholder = Placeholder(window)
        self.slider = Slider(window)
        self.textarea = Textarea(window)

    def load_options(
            self,
            parent_id: str,
            options: Dict[str, Dict[str, Any]]
    ):
        """
        Load options

        :param parent_id: Parent ID
        :param options: Options dict
        """
        for key in options:
            option = options[key]
            self.apply(parent_id, key, option)

    def apply(
            self,
            parent_id: str,
            key: str,
            option: Dict[str, Any]
    ):
        """
        Apply option to field handler based on type

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        """
        if option['type'] == 'int' or option['type'] == 'float':
            if 'slider' in option and option['slider']:
                self.slider.apply(parent_id, key, option)
            else:
                self.input.apply(parent_id, key, option)
        elif option['type'] == 'text':
            self.input.apply(parent_id, key, option)
        elif option['type'] == 'textarea':
            self.textarea.apply(parent_id, key, option)
        elif option['type'] == 'bool':
            self.checkbox.apply(parent_id, key, option)
        elif option['type'] == 'bool_list':
            self.checkbox_list.apply(parent_id, key, option)
        elif option['type'] == 'dict':
            self.dictionary.apply(parent_id, key, option)
        elif option['type'] == 'combo':
            self.combo.apply(parent_id, key, option)
        elif option['type'] == 'cmd':
            self.cmd.apply(parent_id, key, option)

    def apply_value(
            self,
            parent_id: str,
            key: str,
            option: Dict[str, Any],
            value: Any
    ):
        """
        Apply value to option

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        :param value: Option value
        """
        option['value'] = value
        self.apply(parent_id, key, option)

    def get_value(
            self,
            parent_id: str,
            key: str,
            option: Dict[str, Any],
            idx: bool = False
    ) -> Any:
        """
        Get value from field handler based on type

        :param parent_id: Parent ID
        :param key: Option key
        :param option: Option dict
        :param idx: return selected idx, not the value
        :return: Option value
        """
        if option['type'] == 'int' or option['type'] == 'float':
            if 'slider' in option and option['slider']:
                return self.slider.get_value(parent_id, key, option)
            else:
                return self.input.get_value(parent_id, key, option)
        elif option['type'] == 'text':
            return self.input.get_value(parent_id, key, option)
        elif option['type'] == 'textarea':
            return self.textarea.get_value(parent_id, key, option)
        elif option['type'] == 'bool':
            return self.checkbox.get_value(parent_id, key, option)
        elif option['type'] == 'bool_list':
            return self.checkbox_list.get_value(parent_id, key, option)
        elif option['type'] == 'dict':
            return self.dictionary.get_value(parent_id, key, option)
        elif option['type'] == 'combo':
            return self.combo.get_value(parent_id, key, option, idx)
        elif option['type'] == 'cmd':
            return self.cmd.get_value(parent_id, key, option)

    def update_list(
            self,
            option: Dict[str, Any],
            parent_id: str,
            key: str,
            items: List[Dict]
    ):
        """
        Update list items

        :param option: Option dict
        :param parent_id: Parent ID
        :param key: Option key
        :param items: Items dict
        """
        if "type" not in option:
            return
        if option['type'] == 'combo':
            as_dict = {}
            for item in items:
                for k, v in item.items():
                    as_dict[k] = v
            self.update_combo(parent_id, key, as_dict)
        elif option['type'] == 'bool_list':
            self.update_bool_list(parent_id, key, items)

    def update_combo(
            self,
            parent_id: str,
            key: str,
            items: Dict[str, str]
    ):
        """
        Update combo items

        :param parent_id: Parent ID
        :param key: Option key
        :param items: Items dict
        """
        self.combo.update_list(parent_id, key, items)

    def update_bool_list(
            self,
            parent_id: str,
            key: str,
            items: List[Dict]
    ):
        """
        Update combo items

        :param parent_id: Parent ID
        :param key: Option key
        :param items: Items dict
        """
        self.checkbox_list.update_list(parent_id, key, items)
